__all__ = 'QtProjectAttributeInstance'

from PySide2 import QtWidgets

import gm_meta_plugin
from gm_qt_plugin import qt_constants, qt_utils


class QtProjectAttributeInstance(QtWidgets.QWidget, gm_meta_plugin.ProjectAttributeInstance):

    def __init__(self, project_attribute, parent=None):
        QtWidgets.QWidget.__init__(self, parent=parent)
        gm_meta_plugin.ProjectAttributeInstance.__init__(self, project_attribute)

        self._widgets = {}

        widget_id = project_attribute.get_id()
        qt_utils.add_widgets_for_attribute(self.get_plugin(),
                                           project_attribute.get_attribute(),
                                           self._widgets,
                                           widget_id)

    def get_widget_id(self, is_remote=False):
        if is_remote:
            return self.get_id() + qt_constants.REMOTE_SOURCE_SUFFIX
        else:
            return self.get_id()

    def get_widget(self, is_remote=False):

        try:
            # not all project attributes have remote widgets so this may fail with a key error
            widget = self._widgets[self.get_widget_id(is_remote=is_remote)]
        except KeyError as e:
            # if it does fail just return the non remote version
            widget = self._widgets[self.get_widget_id(is_remote=False)]

        return widget

    def get_value(self, widgets=None, is_remote=False):
        return self.get_widget(is_remote=is_remote).get_value()

    def set_value(self, value, widgets=None, is_remote=False):
        return self.get_widget(is_remote=is_remote).set_value(value)

    """
    @staticmethod
    def draw_project_attributes(form_layout,
                                project_attribute,
                                widgets,
                                draw_linked_attributes=True,
                                remote_source=False,
                                encountered_transition_error=False):
        '''
        :return: Returns False if no transition error was encountered otherwise returns True
        :rtype: bool
        '''
        from gm_qt_plugin.widgets.validated_widget import ValidatedWidget

        attribute = project_attribute.get_attribute()

        # Null attributes represent a base case
        if attribute.get_type() == meta_plugin.AttributeType.NULL:
            return encountered_transition_error

        widget_id = project_attribute.get_widget_id(is_remote=isinstance)
        widget = project_attribute.get_widget(widgets=widgets, is_remote=remote_source)
        value = project_attribute.get_value(widgets=widgets, is_remote=remote_source)

        validated_widget = ValidatedWidget(widget)
        validated_widget.set_to_valid()

        # Store the validated widget in the widgets dictionary so PySide does not garbage collect it
        widgets[widget_id + "_validated"] = validated_widget

        try:
            # attempt to get the next project attribute
            next_project_attribute = project_attribute.transition(value)
            transition_error_message = None

        except meta_plugin.errors.RejectedTransitionInputError as e:
            # if there is a transition error set alert to True so that the app renders the current input in red
            validated_widget.set_to_invalid()
            transition_error_message = e.user_message

            # if it is possible to force the transition, then do so and continue as normal
            try:
                next_project_attribute = project_attribute.transition(value, force_transition=True)
                encountered_transition_error = True
            except meta_plugin.errors.RejectedTransitionInputError as e:
                # otherwise it is not possible to know which project attribute to render next so we must raise the error
                return True

        # Add the input to the layout
        form_layout.addRow(attribute.get_display_name(), validated_widget)

        # todo draw description
        '''
        # draw attribute description
        if attribute.get_description():
            description_layout = col2.row()
            description_layout.label(text=attribute.get_description())
            description_layout.enabled = False
        '''

        # draw transition error message
        if transition_error_message:
            validated_widget.set_warning_message(transition_error_message)

        if next_project_attribute and draw_linked_attributes:
            return QtProjectAttributeInstance.draw_project_attributes(form_layout,
                                                                      next_project_attribute,
                                                                      widgets,
                                                                      encountered_transition_error=encountered_transition_error,
                                                                      remote_source=remote_source)

        return encountered_transition_error
    """
